﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.IO.Ports;
using SerialPortConsole.Properties;
using System.Threading;

namespace SerialPortConsole
{
    class Program
    {
        const string RDY_STRING = "RDY";

        static SerialPort serialPort;
        static Boolean ready = false;
        static Boolean process = true;


        static void Main(string[] args)
        {
            try
            {
                serialPort = SerialPortFactory.CreateSerialPort(Settings.Default.Port, Settings.Default.Baud);

                Console.WriteLine("Open Serial Port");
                serialPort.Open();
                Console.WriteLine("Serial Port is open {0}", serialPort.IsOpen);
                if (serialPort.IsOpen)
                {
                    Console.WriteLine("Port: {0}", serialPort.PortName);
                    Console.WriteLine("Baud: {0}", serialPort.BaudRate);
                }


                serialPort.DataReceived += new SerialDataReceivedEventHandler(sp_DataReceived);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }


            while (process)
            {
                //Thread.Sleep(500); 
                if (ready)
                {
                    Console.WriteLine("Send servo command");
                    ready = false;
                }
            }
            

        }

        static void sp_DataReceived(Object sender, EventArgs e)
        {
            try
            {
                
                byte[] buff = new byte[serialPort.BytesToRead];
                int byteCount = serialPort.Read(buff, 0, serialPort.BytesToRead);
                ParseAsii(ref buff, byteCount);

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }


        private static void ParseAsii(ref byte[] buff, int byteCount)
        {
            Encoding ascii = Encoding.ASCII;
            char[] asciiChars = new char[ascii.GetCharCount(buff, 0, byteCount)];
            ascii.GetChars(buff, 0, buff.Length, asciiChars, 0);
            string asciiString = new string(asciiChars);

            Console.WriteLine(asciiString);

            if (asciiString.Contains(RDY_STRING))
                ready = true;
        }



    }



    public class SerialPortFactory 
    {

        SerialPort sp { get; set; }

        public SerialPortFactory(string port, int baud)
        {
            if (string.IsNullOrEmpty(port))
                throw new ArgumentNullException(@"The port can not be empty or null");
            else
                port = port.ToUpper();

            if (!port.Contains("COM"))
                throw new ArgumentException(@"The port argument must be COM followed by an integer.  For example COM2.");

            if(baud <= 0)
                throw new ArgumentException(@"The baud must be a positive integer.");

            sp = new SerialPort(port, baud);        
        }


        public static SerialPort CreateSerialPort(string port, int baud)
        {
            SerialPortFactory s = new SerialPortFactory(port, baud);
            return s.sp;
        }

    }
}
